
(* Analyseur lexical pour Arith *)

{
  open Lexing
  open Parser
   
  exception Lexing_error of char
    
  let keywordsList = ["else",ELSE;  "end",END;  "false",BOOL false;  "if",IF;  "in",IN;  "let",LET;  "match",MATCH;  "mod",MOD;  "module",MODULE;  "open",OPEN;  "rec",REC;  "sig",SIG;  "struct",STRUCT;  "then",THEN;  "true",BOOL true;  "val",VAL;  "with",WITH]
  let keywords = Hashtbl.create (List.length keywordsList)
  let () = List.iter (fun (s,ts) -> Hashtbl.add keywords s ts) keywordsList

  let id_or_kwd s = try Hashtbl.find keywords s with _ -> LIDENT s

  let newline lexbuf =
    let pos = lexbuf.lex_curr_p in
    lexbuf.lex_curr_p <- 
      { pos with pos_lnum = pos.pos_lnum + 1; pos_bol = pos.pos_cnum }

}

let alpha = ['a'-'z' 'A'-'Z']
let chiffre = ['0'-'9']
let lident = ['a'-'z'] (alpha | '_' | '\'' | chiffre)*
let uident = ['A'-'Z'] (alpha | '_' | '\'' | chiffre)*
let entier = ['0'-'9']+
let space = [' ' '\t']

rule token = parse
  | '\n'    { newline lexbuf; token lexbuf }
  | space+  { token lexbuf }
  | "(*"    { comment lexbuf; token lexbuf }

  | lident as id { id_or_kwd id }
  | uident as id { UIDENT id }
  | entier as s  { INT (int_of_string s) }
  
  | '+'     { PLUS }
  | '-'     { MINUS }
  | '*'     { TIMES }
  | '/'     { DIV }

  | '='     { EQ }
  | "<>"    { COMP "<>" }
  | '<'     { COMP "<" }
  | "<="    { COMP "<=" }
  | '>'     { COMP ">" }
  | ">="    { COMP ">=" }

  | "&&"    { AND }
  | "||"    { OR }

  | '('     { LP }
  | '['     { LSB }
  | ')'     { RP }
  | ']'     { RSB }

  | ';'     { SEMI }
(*| ";;"    { SEMI2 }*)
  | ':'     { COLON }
  | "::"    { COLON2 }

  | ":="    { COLONEQ }
  | '.'     { PT }
  | '!'     { EXCL }
  | '|'     { BAR }
  | "->"    { MINUSGT }
  
  | eof     { EOF }
  | _ as c  { raise (Lexing_error c) }

and comment = parse
  | "*)" {  () }
  | "(*" {  comment lexbuf; comment lexbuf }
  | _     { comment lexbuf }
  | eof   { failwith "commentaire non terminé" }
