
%{
  open Ast
  open Parser2

  let make_elt sp ep e   = {elt_desc = e;elt_loc=(sp,ep)}
  let make_expr sp ep e  = {expr_desc = e;expr_loc=(sp,ep)}
  let make_decl sp ep e  = {decl_desc = e;decl_loc=(sp,ep)}
  let make_mexpr sp ep e = {mexpr_desc = e;mexpr_loc=(sp,ep)}

%}

/* Notice that [ ] is like [] and ( ) is like () */

%token <int> INT
%token <string> LIDENT
%token <string> UIDENT
%token <bool> BOOL
%token PLUS, MINUS, TIMES, DIV, MOD
%token <string> COMP
%token EQ
%token AND, OR /* && || */

%token LP, RP, LSB, RSB /* l : left, r : right, p : parenthesis, sb : square bracket */
%token SEMI, (*SEMI2,*) COLON, COLON2 /* ; ;; : :: */
%token COLONEQ, PT, EXCL, BAR, MINUSGT /* := . ! | -> */

%token IF, THEN, ELSE
%token LET, IN, REC
%token MATCH, MODULE, OPEN, SIG, STRUCT, VAL, WITH, END
%token EOF 

/* Définitions des priorités et associativités des tokens */

/* VERIFIER LA PLACE DE MATCH/WITH */

%nonassoc IN
%nonassoc MATCH
%left SEMI
%nonassoc THEN
%nonassoc ELSE
%right COLONEQ
%left OR
%left AND
%left COMP EQ
%right COLON2
%left PLUS MINUS 
%left TIMES DIV MOD
%nonassoc uminus

/* Point d'entrée de la grammaire */
%start prog

/* Type des valeurs retournées par l'analyseur syntaxique */
%type <Ast.program> prog

%%

prog:
| p = elt* EOF                   { p }
;

elt:
| LET LP RP EQ e=expr            { make_elt $startpos $endpos (LetIgnore e) }
| LET b=boption(REC) s=LIDENT f=farg* COLON t=type_menhir EQ e=expr
                                 { make_elt $startpos $endpos (Let (b,s,f,t,e)) }
| MODULE id=UIDENT m=marg* s=semisignature? EQ e=mexpr
                                 { make_elt $startpos $endpos (Module (id,m,s,e)) }
| OPEN s=upath                   { make_elt $startpos $endpos (Open s) }
;
 
simple_expr:
| LP e = expr RP                 { e }
| c=cst                          { make_expr $startpos $endpos (Cst c) }
| id=lpath                       { make_expr $startpos $endpos (Var id) }
| EXCL e=simple_expr             { make_expr $startpos $endpos (Unop (Excl, e)) }
;

expr:
| e=simple_expr                  { e }
| f=lpath args=simple_expr+      { make_expr $startpos $endpos (App(f,args)) }
| e1=expr o=op e2=expr           { make_expr $startpos $endpos (Binop (o,e1,e2)) }
| MINUS e = expr %prec uminus    { make_expr $startpos $endpos (Unop (Minus, e)) }
| IF e1=expr THEN e2=expr        { make_expr $startpos $endpos (IfThenElse (e1,e2,make_expr $startpos $endpos (Cst Unit))) }
| IF e1=expr THEN e2=expr ELSE e3=expr
                                 { make_expr $startpos $endpos (IfThenElse (e1,e2,e3)) }
| LET id = LIDENT EQ e1 = expr IN e2 = expr 
                                 { make_expr $startpos $endpos (LetIn (id,e1,e2)) }
| MATCH e=expr WITH BAR? LSB RSB MINUSGT e1=expr
BAR x=LIDENT COLON2 s=LIDENT MINUSGT e2=expr %prec MATCH
                                 { make_expr $startpos $endpos (MatchWith (e,e1,x,s,e2)) }
;

signature:
| SIG s=decl* END                { s }

%inline semisignature:
| COLON s=signature              { s }

decl:
| VAL id=LIDENT COLON lt=separated_nonempty_list(MINUSGT,type_menhir)
                                 { make_decl $startpos $endpos (Val (id,lt)) }
| MODULE id=UIDENT m=marg* COLON s=signature
                                 { make_decl $startpos $endpos (ModuleS (id,m,s)) }

mexpr:
| STRUCT el=elt* END             { make_mexpr $startpos $endpos (Struct el) }
| id=upath l=list(functor_arg)   { make_mexpr $startpos $endpos (Functor (id,l)) }

%inline functor_arg:
| LP id=upath RP                 { id }

%inline farg:
| LP id=LIDENT COLON t=type_menhir RP     { (id,t) }
;

%inline marg :
| LP id=UIDENT COLON s=signature RP       { (id,s) }
;

%inline type_menhir:
| t=LIDENT+                      { stringToTyp $startpos $endpos (List.rev t) }

upath:
| id=UIDENT                      { [id] }
| id=UIDENT PT p=upath           { id::p }
;

lpath:
| id=LIDENT                      { [id] }
| id=UIDENT PT p=lpath           { id::p }
;

%inline cst:
| i=INT                          { Int i }
| b=BOOL                         { Bool b }
| LP RP                          { Unit }
| LSB RSB                        { EmptyList }
;

%inline op:
| PLUS   { Add }
| MINUS  { Sub }
| TIMES  { Mul }
| DIV    { Div }
| MOD    { Mod }
| s=COMP { Comp (stringToComp s) }
| EQ     { Comp Eq }
| AND    { And }
| OR     { Or }
| COLON2 { Concat }
| COLONEQ{ Assert }
| SEMI   { Semi}
;


